# -*- coding: utf-8 -*-

from kotti import DBSession
from kotti.resources import Document
from kotti.resources import File
from kotti.resources import TypeInfo
from kotti.security import view_permitted
from kotti.util import _
from kotti.util import ViewLink
from kotti.resources import Image
from pprint import pformat
from sqlalchemy import Column
from sqlalchemy import ForeignKey
from sqlalchemy import Integer


class VideoFileTypeInfo(TypeInfo):

    def addable(self, context, request):
        """Return True if
            - the type described in 'self' may be added  *and*
            - no other child of the same type has already be added
           to 'context'."""

        if view_permitted(context, request, self.add_view):
            addable = context.type_info.name in self.addable_to
            child_type_already_added = self in [c.type_info for c in context.children]
            return addable and not child_type_already_added
        else:
            return False

    def copy(self, **kwargs):

        d = self.__dict__.copy()
        d.update(kwargs)
        return VideoFileTypeInfo(**d)

    def __repr__(self):

        return pformat(self.__dict__)


generic_video_file_type_info = VideoFileTypeInfo(name=u"VideoFile",
                                                 title=_(u"Video file"),
                                                 addable_to=[u"Video", ],
                                                 add_view=None,
                                                 edit_links=[ViewLink('edit', title=_(u'Edit')), ], )


class Mp4File(File):

    id = Column(Integer(), ForeignKey('files.id'), primary_key=True)

    type_info = generic_video_file_type_info.copy(name=u"Mp4File",
                                                  title=_(u"Video file (*.mp4)"),
                                                  add_view="add_mp4file")

    def __init__(self, data=None, filename=None, mimetype=None, size=None, **kwargs):
        super(Mp4File, self).__init__(data=data, filename="video.mp4", mimetype="video/mp4", size=size, **kwargs)


class WebmFile(File):

    id = Column(Integer(), ForeignKey('files.id'), primary_key=True)

    type_info = generic_video_file_type_info.copy(name=u"WebmFile",
                                                  title=_(u"Video file (*.webm)"),
                                                  add_view="add_webmfile")

    def __init__(self, data=None, filename=None, mimetype=None, size=None, **kwargs):
        super(WebmFile, self).__init__(data=data, filename="video.webm", mimetype="video/webm", size=size, **kwargs)


class OggFile(File):

    id = Column(Integer(), ForeignKey('files.id'), primary_key=True)

    type_info = generic_video_file_type_info.copy(name=u"OggFile",
                                                  title=_(u"Video file (*.ogg)"),
                                                  add_view="add_oggfile")

    def __init__(self, data=None, filename=None, mimetype=None, size=None, **kwargs):
        super(OggFile, self).__init__(data=data, filename="video.ogv", mimetype="video/ogg", size=size, **kwargs)


class SubtitlesFile(File):

    id = Column(Integer(), ForeignKey('files.id'), primary_key=True)

    type_info = generic_video_file_type_info.copy(name=u"SubtitlesFile",
                                                  title=_(u"Subtitles file (*.srt)"),
                                                  add_view="add_subtitlesfile")


class ChaptersFile(File):

    id = Column(Integer(), ForeignKey('files.id'), primary_key=True)

    type_info = generic_video_file_type_info.copy(name=u"ChaptersFile",
                                                  title=_(u"Chapters file (*.srt)"),
                                                  add_view="add_chaptersfile")


class Video(Document):

    id = Column(Integer(), ForeignKey('documents.id'), primary_key=True)

    type_info = Document.type_info.copy(name=u"Video",
                                        title=_(u"Video"),
                                        addable_to=[u"Document"],
                                        add_view="add_video", )

    @property
    def mp4_file(self):

        session = DBSession()
        query = session.query(Mp4File).filter(Mp4File.parent_id == self.id)

        if query.count() > 0:
            return query.first()

        return None

    @property
    def webm_file(self):

        session = DBSession()
        query = session.query(WebmFile).filter(WebmFile.parent_id == self.id)

        if query.count() > 0:
            return query.first()

        return None

    @property
    def ogg_file(self):

        session = DBSession()
        query = session.query(OggFile).filter(OggFile.parent_id == self.id)

        if query.count() > 0:
            return query.first()

        return None

    @property
    def subtitles_file(self):

        session = DBSession()
        query = session.query(SubtitlesFile).filter(SubtitlesFile.parent_id == self.id)

        if query.count() > 0:
            return query.first()

        return None

    @property
    def chapters_file(self):

        session = DBSession()
        query = session.query(ChaptersFile).filter(ChaptersFile.parent_id == self.id)

        if query.count() > 0:
            return query.first()

        return None

    @property
    def poster_file(self):

        session = DBSession()
        query = session.query(Image).filter(Image.parent_id == self.id)

        if query.count() > 0:
            return query.first()

        return None



# -*- coding: utf-8 -*-

import logging
log = logging.getLogger(__name__)


def kotti_configure(settings):

    settings['kotti.includes'] += ' kotti_video.views'
    settings['kotti.available_types'] += ' kotti_video.resources.Video'
    settings['kotti.available_types'] += ' kotti_video.resources.Mp4File'
    settings['kotti.available_types'] += ' kotti_video.resources.WebmFile'
    settings['kotti.available_types'] += ' kotti_video.resources.OggFile'
    settings['kotti.available_types'] += ' kotti_video.resources.SubtitlesFile'
    settings['kotti.available_types'] += ' kotti_video.resources.ChaptersFile'



from setuptools import setup, find_packages
import os

version = '0.2'

here = os.path.abspath(os.path.dirname(__file__))

try:
    README = open(os.path.join(here, 'README.rst')).read()
except IOError:
    README = ""
try:
    TODO = open(os.path.join(here, 'TODO.txt')).read()
except IOError:
    TODO = ""
try:
    CHANGES = open(os.path.join(here, 'CHANGES.txt')).read()
except IOError:
    CHANGES = ""

tests_require = ["pytest",
                 "WebTest",
                 "wsgi_intercept",
                 "zope.testbrowser", ]

setup(name='kotti_video',
      version=version,
      description="Add videos to your Kotti site",
      long_description=README + '\n\n' + TODO + '\n\n' + CHANGES,
      classifiers=["Programming Language :: Python",
                   "Framework :: Pylons",
                   "Topic :: Internet :: WWW/HTTP",
                   "Topic :: Internet :: WWW/HTTP :: Dynamic Content",
                   "Topic :: Internet :: WWW/HTTP :: WSGI :: Application",
                   "License :: Repoze Public License", ],
      keywords='video medialementjs kotti cms pylons pyramid',
      author='Andreas Kaiser',
      author_email='disko@binary-punks.com',
      url='https://github.com/disko/kotti_video',
      license='BSD-derived (http://www.repoze.org/LICENSE.txt)',
      packages=find_packages(exclude=['ez_setup', 'examples', 'tests']),
      include_package_data=True,
      zip_safe=False,
      install_requires=["Kotti>=0.7dev", ],
      tests_require=tests_require,
      extras_require={'testing': tests_require, }, )



# -*- coding: utf-8 -*-

import logging
from kotti.util import _
from kotti.views.edit import DocumentSchema, make_generic_add, make_generic_edit
from kotti.views.file import AddFileFormView, EditFileFormView
from kotti_video.resources import Video, Mp4File, WebmFile, OggFile, SubtitlesFile, ChaptersFile
from pyramid.response import Response
from pyramid.url import resource_url
from pyramid.view import view_config

log = logging.getLogger(__name__)


class BaseView(object):

    def __init__(self, context, request):

        self.context = context
        self.request = request


class VideoView(BaseView):

    @view_config(context=Video,
                 name='view',
                 permission='view',
                 renderer='templates/video-view.pt')
    def view(self):
        result = {}
        for t in ("mp4", "webm", "ogg", "subtitles", "chapters", "poster"):
            key = "%s_url" % t
            file = getattr(self.context, "%s_file" % t)
            if file is None:
                result[key] = None
            else:
                result[key] = resource_url(file, self.request, "@@attachment-view")

        return result


class AddMp4FileFormView(AddFileFormView):

    item_type = _(u"Mp4File")
    item_class = Mp4File


class AddWebmFileFormView(AddFileFormView):

    item_type = _(u"WebmFile")
    item_class = WebmFile


class AddOggFileFormView(AddFileFormView):

    item_type = _(u"OggFile")
    item_class = OggFile


class AddSubtitlesFileFormView(AddFileFormView):

    item_type = _(u"SubtitlesFile")
    item_class = SubtitlesFile


class AddChaptersFileFormView(AddFileFormView):

    item_type = _(u"ChaptersFile")
    item_class = ChaptersFile


def includeme(config):

    config.add_static_view('static-kotti_video', 'kotti_video:static')
    config.scan("kotti_video")

    # Video add/edit
    config.add_view(make_generic_add(DocumentSchema(), Video),
                                     name=Video.type_info.add_view,
                                     permission='add',
                                     renderer='kotti:templates/edit/node.pt', )
    config.add_view(make_generic_edit(DocumentSchema()),
                    context=Video,
                    name='edit',
                    permission='edit',
                    renderer='kotti:templates/edit/node.pt', )

    # VideoFile types edit
    for file_type in (Mp4File, WebmFile, OggFile, SubtitlesFile, ChaptersFile):
        config.add_view(EditFileFormView,
                        context=file_type,
                        name='edit',
                        permission='edit',
                        renderer='kotti:templates/edit/node.pt', )
    # Mp4File add
    config.add_view(AddMp4FileFormView,
                    name=Mp4File.type_info.add_view,
                    permission='add',
                    renderer='kotti:templates/edit/node.pt', )
    # WebmFile add
    config.add_view(AddWebmFileFormView,
                    name=WebmFile.type_info.add_view,
                    permission='add',
                    renderer='kotti:templates/edit/node.pt', )
    # OggFile add
    config.add_view(AddOggFileFormView,
                    name=OggFile.type_info.add_view,
                    permission='add',
                    renderer='kotti:templates/edit/node.pt', )
    # SubtitlesFile add
    config.add_view(AddSubtitlesFileFormView,
                    name=SubtitlesFile.type_info.add_view,
                    permission='add',
                    renderer='kotti:templates/edit/node.pt', )
    # ChaptersFile add
    config.add_view(AddChaptersFileFormView,
                    name=ChaptersFile.type_info.add_view,
                    permission='add',
                    renderer='kotti:templates/edit/node.pt', )
